# Reference

<!-- DO NOT EDIT: This document was generated by Puppet Strings -->

## Table of Contents

### Classes

* [`sap`](#sap): Installs prerequisites for an SAP environment on RedHat derivatives
* [`sap::cluster`](#sapcluster): Configures a node with the appropriate SAP cluster resources
* [`sap::cluster::prepare`](#sapclusterprepare): Ensure local SAP configuration is ready for a clustered environment
* [`sap::config`](#sapconfig): Private class used to control configuration file deployment.
* [`sap::config::common`](#sapconfigcommon): Trick SAP into thinking this host is running a supported RHEL
* [`sap::config::limits`](#sapconfiglimits): Generates an etc limits file for each relevant enabled component.
* [`sap::config::mount_points`](#sapconfigmount_points): Creates and manages mountpoints relevant to each instance type.
* [`sap::config::router`](#sapconfigrouter): Class: sap::config::router  This class contain the configuration for SAP Router  Parameters:   This module has no parameters  Actions:      T
* [`sap::config::sysctl`](#sapconfigsysctl): Configures sysctl parameters for the selected components.
* [`sap::config::tmpfs`](#sapconfigtmpfs): Ensures that the tmpfs partition is configured per SAP recommendations Add swap space check per
* [`sap::install`](#sapinstall): Installs the packages associated with the selected sets
* [`sap::install::mount_dependencies`](#sapinstallmount_dependencies): Installs dependent modules needed for mount points
* [`sap::management`](#sapmanagement): Manages configuration post install of SAP.
* [`sap::management::ads_fonts`](#sapmanagementads_fonts): Deploys font and XDC files to ADS systems
* [`sap::management::db2_queries`](#sapmanagementdb2_queries): Implementation detail - creates sql query scheduling infrastructure
* [`sap::management::dotfiles`](#sapmanagementdotfiles): Manages the dotfiles belonging to SAP users.
* [`sap::management::etc_services`](#sapmanagementetc_services): Creates all relevant etc services entries for this system
* [`sap::management::firewall`](#sapmanagementfirewall): Creates all relevant firewall rules for this system
* [`sap::management::permissions`](#sapmanagementpermissions): Implementation detail - corrects permissions as needed
* [`sap::management::profiles`](#sapmanagementprofiles): Creates the profiles files in /sapmnt/SID/profile for each SID
* [`sap::params`](#sapparams): This module contains the parameters for SAP Netweaver
* [`sap::service`](#sapservice): Configures OS level services required for SAP Netweaver instances
* [`sap::service::cloudconnector`](#sapservicecloudconnector): SAP Cloud Connector service configuration
* [`sap::service::router`](#sapservicerouter): Configures the SAP Router service

### Defined types

* [`sap::cluster::as`](#sapclusteras)
* [`sap::cluster::as_inst`](#sapclusteras_inst): Creates Corosync/Pacemaker cluster rules for a single AS instance.
* [`sap::cluster::db2`](#sapclusterdb2): Creates Corosync/Pacemaker cluster rules for a SAP DB2 HA/DR instance.
* [`sap::cluster::fence_lpar`](#sapclusterfence_lpar): Creates Corosync/Pacemaker rules for a fence_lpar STONITH device
* [`sap::cluster::filesystem`](#sapclusterfilesystem): Creates Corosync/Pacemaker cluster rules for a filesystem resource.
* [`sap::cluster::lvm`](#sapclusterlvm): Creates Corosync/Pacemaker cluster rules for an LVM resource.
* [`sap::cluster::scsers`](#sapclusterscsers): Creates Corosync/Pacemaker cluster rules for an (A)SCS/ERS instance.
* [`sap::config::mount_point`](#sapconfigmount_point): Declares a single directory or mountpoint.
* [`sap::management::db2_query_schedule`](#sapmanagementdb2_query_schedule): Implementation detail - a query file & associate cron schedule

### Functions

* [`multiregsubst`](#multiregsubst): Performs a series of substitutions on the same target string
* [`sap::cluster::filter_nodes`](#sapclusterfilter_nodes): Simple function to retrieve only the nodes with a desired role
* [`sap::sysctl_calculated_values`](#sapsysctl_calculated_values): Recursive function used to compute the value for a sysctl var.
* [`sap::sysctl_line`](#sapsysctl_line): This function converts a sysctl entry hash into a string line.

### Data types

* [`Sap::ABAPLang`](#sapabaplang)
* [`Sap::ABAPLoginGroup`](#sapabaplogingroup)
* [`Sap::ABAPPassword`](#sapabappassword): Ideally: type Sap::ABAPPassword = Sensitive[Pattern[/\A[[:graph:]]{1,40}\z/]]
* [`Sap::ABAPUser`](#sapabapuser)
* [`Sap::AbapClient`](#sapabapclient)
* [`Sap::BackendDatabase`](#sapbackenddatabase): Constrained list of backend databases SAP might connect to
* [`Sap::ClusterAs`](#sapclusteras): Used to model linux high availability rules for one or more application server instances. Note that this does not support the 'primary applic
* [`Sap::ClusterComponent`](#sapclustercomponent): Higher level model for linux high-availability rules which captures the type and any subcomponents of each component
* [`Sap::ClusterComponentType`](#sapclustercomponenttype): Constrains the list of valid resource types
* [`Sap::ClusterDb2`](#sapclusterdb2): Used to model linux high availability rules for a HADR based DB2 cluster Note that this assumes a single SID per instance. (SAP typically ass
* [`Sap::ClusterDbMode`](#sapclusterdbmode)
* [`Sap::ClusterFence`](#sapclusterfence)
* [`Sap::ClusterFenceData`](#sapclusterfencedata)
* [`Sap::ClusterFenceDataLpar`](#sapclusterfencedatalpar)
* [`Sap::ClusterFenceHostMap`](#sapclusterfencehostmap)
* [`Sap::ClusterFenceType`](#sapclusterfencetype)
* [`Sap::ClusterFilesystem`](#sapclusterfilesystem)
* [`Sap::ClusterLvm`](#sapclusterlvm): Parameters for an LVM volume
* [`Sap::ClusterNodeRoles`](#sapclusternoderoles): Mapping from each node to an array listing all of the roles this node fulfills within this cluster. For example {   'pr0-ap00' => [ 'app-stan
* [`Sap::ClusterScsErs`](#sapclusterscsers): Used to model linux high availability rules for an (A)SCS/ERS pair
* [`Sap::Db2QuerySchedule`](#sapdb2queryschedule)
* [`Sap::Db2Schema`](#sapdb2schema)
* [`Sap::DbHaMode`](#sapdbhamode)
* [`Sap::FirewallRule`](#sapfirewallrule)
* [`Sap::FirewallRulePort`](#sapfirewallruleport)
* [`Sap::FirewallServices`](#sapfirewallservices)
* [`Sap::FsType`](#sapfstype): Supported filesystems
* [`Sap::HostRuleEntry`](#saphostruleentry)
* [`Sap::InstClass`](#sapinstclass)
* [`Sap::InstClassAux`](#sapinstclassaux)
* [`Sap::InstClassDb`](#sapinstclassdb)
* [`Sap::InstClassSap`](#sapinstclasssap)
* [`Sap::InstClassSapApp`](#sapinstclasssapapp)
* [`Sap::InstClassSapCs`](#sapinstclasssapcs)
* [`Sap::InstData`](#sapinstdata): Simple structure containing the type of an instance along with it's instance number and the host name of the server on which it must run. Thi
* [`Sap::InstNumber`](#sapinstnumber)
* [`Sap::InstType`](#sapinsttype)
* [`Sap::MessageServer`](#sapmessageserver): Describes the message server information needed for an external connection to a given SID.
* [`Sap::MountPointSIDMode`](#sapmountpointsidmode)
* [`Sap::MountPoints`](#sapmountpoints)
* [`Sap::NodeRole`](#sapnoderole)
* [`Sap::PuppetUrl`](#sappuppeturl)
* [`Sap::Reginfo`](#sapreginfo)
* [`Sap::SID`](#sapsid): Format of an SAP system ID
* [`Sap::SIDClusterData`](#sapsidclusterdata)
* [`Sap::SIDConfigEntry`](#sapsidconfigentry): Configuration entry for a SID
* [`Sap::SIDDatabase`](#sapsiddatabase)
* [`Sap::SIDDatabaseDb2`](#sapsiddatabasedb2): Hash describing an instance which is part of a given SAP SID.
* [`Sap::SIDInstance`](#sapsidinstance): Structure detailing the attributes of a given SAP instance class. This is assigned as a component of a specific SID's sidconfigentry.  Note t
* [`Sap::SapComponent`](#sapsapcomponent): Structure representing each valid component which could be installed on an SAP server. The * `ads`            - Adobe Document Services * `ba
* [`Sap::ScheduleEntry`](#sapscheduleentry)
* [`Sap::SecRuleMode`](#sapsecrulemode)
* [`Sap::SecUser`](#sapsecuser)
* [`Sap::Secinfo`](#sapsecinfo)
* [`Sap::SourceURL`](#sapsourceurl)
* [`Sap::SourcedFile`](#sapsourcedfile)
* [`Sap::TargetProgram`](#saptargetprogram)
* [`Sap::VgLvName`](#sapvglvname): This artificially restricts vg names to [a-zA-Z0-9_]{2,} for simplification purposes.

## Classes

### <a name="sap"></a>`sap`

This module manages SAP prerequisites for several types of SAP installations
based on corresponding SAP OSS notes

#### Examples

##### Application server containing an ADS instance and a Netweaver instance

```puppet
class { 'sap':
  system_ids       => {
    'PR0'          => {
      'components'       => [
         'base',
         'base-extended',
         'ads',
      ],
      'backend_database' => [ 'db2' ],
    },
  },
}
```

#### Parameters

The following parameters are available in the `sap` class:

* [`system_ids`](#system_ids)
* [`pattern_map`](#pattern_map)
* [`create_mount_points`](#create_mount_points)
* [`mount_points`](#mount_points)
* [`router_oss_realm`](#router_oss_realm)
* [`manage_mount_dependencies`](#manage_mount_dependencies)
* [`router_rules`](#router_rules)
* [`distro_text`](#distro_text)
* [`component_packages`](#component_packages)
* [`backenddb_packages`](#backenddb_packages)
* [`message_servers`](#message_servers)
* [`bash_source_profile`](#bash_source_profile)
* [`manage_firewall`](#manage_firewall)
* [`deployed_fonts`](#deployed_fonts)
* [`deployed_xdc`](#deployed_xdc)

##### <a name="system_ids"></a>`system_ids`

Data type: `Hash[Sap::SID, Sap::SIDConfigEntry]`

An hash of SAP system IDs (SIDs) which will be present on the target
host along with the following possible attributes for each SID:
* `components` - a list of associated sap applications to enable for this
                 SID (mandatory)
* `backend_databases` - types of databases to which this system connects
                        (mandatory for SIDs with 'base' enabled)
* `manage_profiles` - Flag indicating whether the instance profiles for this
                      should be managed by the module. If specified
                      `instances`, `databases`, and `primary_backend_db`
                      become mandatory parameters!
* `exclude_common_mount_points` - flag indicating if this SID should skip
                                  SID specific entries for the 'common'
                                  mountpoint. Some system types do not have
                                  /sapmnt directories (e.g. a DAA instance
                                  or liveCache)
* `pattern_map` - A hash mapping pattern lookup keys to regular expressions
                  which are unique to this SID. Note that if one is added it
                  must have a corresponding value added in `pattern_values`
* `pattern_values` - Mapping from a pattern lookup key to the value which
                     should be provided wherever the corresponding pattern
                     is found.
* `instances` - Hash detailing the instances which are part of this SID.
                Entries are keyed by `instance class` and contain the
                corresponding Sap::SidInstance detail. See [Data Types in
                the readme](./README.md#data-types) for detail.
* `databases` - Hash detailing the database parameters for this SID.
                Entries in this hash are keyed by `instance class`. For
                detail, see [Data Types in the readme](./README.md#data-types)
* `primary_backend_db` - Sap::InstClassDb of the primary backend database
                         for this node
* `default_pfl_entries` - Hash of profile keys and the desired value which
                          will be appended to the end of the DEFAULT.PFL
                          file for this instance. Currently not validated.
* `sec_info_rules` - Array of Sap::SecInfo entries which will be inserted
                     before the final rules in the secinfo file.
* `reg_info_rules` - Array of Sap::RegInfo entries which will be inserted
                     before the final rules in the reginfo file.
* `db2_query_schedules` - Array of Sap::Db2QuerySchedule entries describing
                          a query script file, associated parameters, and
                          the desired cron schedule entry for each. These
                          queries are then configured to execute as the
                          DB2 instance owner on the host system.

Note that each entry must be exactly 3 characters in length and contain
exclusively uppercase characters. Also, if `base` is included in the
components list for an SID at least one backend_database entry must be
provided.

##### <a name="pattern_map"></a>`pattern_map`

Data type: `Hash[String, String]`

Hash mapping a given global key such as 'sid_upper' to the corresponding
regexp used to substitute this value in standard strings. See the
mount_point defined type for detail. Note that the values in this hash
will be converted to regular expressions via `Regexp($value)`

##### <a name="create_mount_points"></a>`create_mount_points`

Data type: `Boolean`

Indicates whether the standard mount points should be created for the
specified instance.

Default value: ``false``

##### <a name="mount_points"></a>`mount_points`

Data type: `Hash[Sap::SapComponent, Sap::MountPoints]`

Defines the mount points and supporting directories which should be created
for each component type. Note that this structure is a deep hash with a `--`
knockout value.

Default value: `{}`

##### <a name="router_oss_realm"></a>`router_oss_realm`

Data type: `Optional[String]`

Specify OSS realm for SAP router connection. For example,
`'p:CN=hostname.domain.tld, OU=0123456789, OU=SAProuter, O=SAP, C=DE'`

Default value: ``undef``

##### <a name="manage_mount_dependencies"></a>`manage_mount_dependencies`

Data type: `Boolean`

When enabled this module will install and configure the puppet-nfs module
detailed here: https://forge.puppet.com/derdanne/nfs
Note that currently only NFSv4 is supported for clients.

Default value: ``false``

##### <a name="router_rules"></a>`router_rules`

Data type: `Optional[Array[String]]`

Specify array of rules for the SAP router

Default value: ``undef``

##### <a name="distro_text"></a>`distro_text`

Data type: `Optional[String]`

Modify text in /etc/redhat-release

Default value: ``undef``

##### <a name="component_packages"></a>`component_packages`

Data type: `Hash[Sap::SapComponent, Array[String]]`

Hash mapping each component type to an array containing the names of the
requried packages on this operating system.

Default value: `{}`

##### <a name="backenddb_packages"></a>`backenddb_packages`

Data type: `Hash[Sap::BackendDatabase, Array[String]]`

Hash mapping each backend database type to an array containing the names
of the requried packages on this operating system.

Default value: `{}`

##### <a name="message_servers"></a>`message_servers`

Data type: `Hash[Sap::SID, Sap::MessageServer]`

Hash detailing the message server connection definitions for other systems
this particular instance will need to communicate with. For entry structure
detail See [Data Types in the readme](./README.md#data-types).

Default value: `{}`

##### <a name="bash_source_profile"></a>`bash_source_profile`

Data type: `Boolean`

When enabled the `.profile` generated by the SAP installer will be sourced
by the .bash_profile of each sidadm user if that file exists.

Default value: ``false``

##### <a name="manage_firewall"></a>`manage_firewall`

Data type: `Boolean`

When enabled all relevant firewall rules for the local system will be
created and managed via firewalld.

Default value: ``false``

##### <a name="deployed_fonts"></a>`deployed_fonts`

Data type: `Sap::SourcedFile`

Hash with a list of fonts to deploy and their source locations. Each file in
this structure will be installed in
`/usr/sap/${SAPSYSTEMNAME}/SYS/global/AdobeDocumentServices/FontManagerService/fonts/customer/`
for each local SID with the ADS component enabled.

Default value: `{}`

##### <a name="deployed_xdc"></a>`deployed_xdc`

Data type: `Sap::SourcedFile`

Hash with a list of xdc files to deploy and their source locations. Each file in
this structure will be installed in
`/usr/sap/${sid}/SYS/global/AdobeDocumentServices/lib/XDC/Customer/`
for each local SID with the ADS component enabled.

Default value: `{}`

### <a name="sapcluster"></a>`sap::cluster`

Cluster class which configures a PCS/Corosync based ha-environment. Note that
it is assumed that the puppet-corosync class is defined and the cluster is
configured before this class is configured.

Additionally, the cluster must be an Asymmetrical "Opt-In" cluster. This
configuration operates under that assumption and sets cluster level parameters
appropriately.

#### Parameters

The following parameters are available in the `sap::cluster` class:

* [`cib_name`](#cib_name)
* [`resource_stickiness`](#resource_stickiness)
* [`enable_fencing`](#enable_fencing)
* [`sid_rules`](#sid_rules)
* [`fence_agents`](#fence_agents)

##### <a name="cib_name"></a>`cib_name`

Data type: `String`

Name of the temporary Cluster Information Base to use during the building
and commiting of the rule changes incorporated in this module.

##### <a name="resource_stickiness"></a>`resource_stickiness`

Data type: `Integer`

Global configuration setting controlling how much a given resource wants to
'stay' where it is as opposed to moving to a 'home' node.

Default value: `500`

##### <a name="enable_fencing"></a>`enable_fencing`

Data type: `Boolean`

Controls whether the cluster assumes Stonith should be enabled or not.

Default value: ``false``

##### <a name="sid_rules"></a>`sid_rules`

Data type: `Sap::SIDClusterData`

A hash mapping each cluster relevant SID to two mandatory components:
*node_roles*: a complete list of PCS nodes mapped to their roles
*components*: cluster elements for this instance with their data

Default value: `{}`

##### <a name="fence_agents"></a>`fence_agents`

Data type: `Array[Sap::ClusterFence]`

An array of Sap::ClusterFence definitions which provide fencing coverage for
all nodes defined by this cluster. Note that this parameter becomes
mandatory when enable_fencing is true. Additionally, every node referenced
within the sid_rules structure must have exactly one corresponding entry in
the pcmk_host_map structure of a single agent defined here.

Default value: `[]`

### <a name="sapclusterprepare"></a>`sap::cluster::prepare`

Simple class which prepares the config files on this machine for a
high-availability configuration of SAP. There are currently two components to
this process: 1) the profile files in /sapmnt/<SID> corresponding to this
machine are updated replacing entries for Restart_Program with Start_Program.
2) All `.sapenv_${HOSTNAME}.(sh|csh)` and other similar host-specific profile
files are removed so the system will only use the `.sapenv.(sh|csh)` variants.

Configuration is performed entirely based on the values of local 'sap' facts.

#### Parameters

The following parameters are available in the `sap::cluster::prepare` class:

* [`cluster_sid_list`](#cluster_sid_list)
* [`packages`](#packages)

##### <a name="cluster_sid_list"></a>`cluster_sid_list`

Data type: `Array[Sap::Sid]`

Cluster configuration steps will be taken for each local instance matching
the provided SID list. If multiple SAP instances are located on a given node
only those which match this list will be updated.

##### <a name="packages"></a>`packages`

Data type: `Array[String]`

Ensures that the resource agents needed for SAP configuration are installed
on this system. Note that defaults are provided only for RHEL 7 derivatives.

Default value: `[]`

### <a name="sapconfig"></a>`sap::config`

Orchestrates the creation of relevant configuration files needed by SAP

### <a name="sapconfigcommon"></a>`sap::config::common`

Overwrites the distro text file to trick SAP into thinking this is RHEL.

### <a name="sapconfiglimits"></a>`sap::config::limits`

This class uses data from hiera to generate an etc limits file for each
enabled component. Note that some components do not specify limits while
others may have SID specific data which will result in a more complicated
limit output

### <a name="sapconfigmount_points"></a>`sap::config::mount_points`

Creates and manages mountpoints relevant to each instance type.

### <a name="sapconfigrouter"></a>`sap::config::router`

Class: sap::config::router

This class contain the configuration for SAP Router

Parameters:   This module has no parameters

Actions:      This module has no actions

Requires:     This module has no requirements

Sample Usage: include sap_router::config

### <a name="sapconfigsysctl"></a>`sap::config::sysctl`

This class configures the sysctl parameters based on the values
provided in sap::param::config_sysctl for each enabled component

### <a name="sapconfigtmpfs"></a>`sap::config::tmpfs`

Ensures that the tmpfs partition is configured per SAP recommendations
Add swap space check per

### <a name="sapinstall"></a>`sap::install`

Packages are installed based on the selections provided to the main sap class
using sane defaults configured in hiera.

### <a name="sapinstallmount_dependencies"></a>`sap::install::mount_dependencies`

When enabled this ensures that all necessary dependencies for mount point
management are installed

### <a name="sapmanagement"></a>`sap::management`

This package manages configuration which is generated during the SAP install.
Note that it must not be applied until after the base 'sap' class.
Additionally, much of the functionality of this subclass is ignored if the
base SAP facts do not seem to indicate that the class has been installed and
configured.

### <a name="sapmanagementads_fonts"></a>`sap::management::ads_fonts`

Deploys font and XDC files to ADS systems

#### Parameters

The following parameters are available in the `sap::management::ads_fonts` class:

* [`system_ids`](#system_ids)
* [`deployed_fonts`](#deployed_fonts)
* [`deployed_xdc`](#deployed_xdc)

##### <a name="system_ids"></a>`system_ids`

Data type: `Hash[Sap::SID, Sap::SIDConfigEntry]`

Sourced directly from the parent system_ids parameter on the SAP class. See
that class for further detail.

Default value: `$sap::system_ids`

##### <a name="deployed_fonts"></a>`deployed_fonts`

Data type: `Sap::SourcedFile`

Hash with a list of fonts to deploy and their source locations. Each file in
this structure will be installed in
`/usr/sap/${SAPSYSTEMNAME}/SYS/global/AdobeDocumentServices/FontManagerService/fonts/customer/`
for each local SID with the ADS component enabled.

Default value: `$sap::deployed_fonts`

##### <a name="deployed_xdc"></a>`deployed_xdc`

Data type: `Sap::SourcedFile`

Hash with a list of xdc files to deploy and their source locations. Each file in
this structure will be installed in
`/usr/sap/${sid}/SYS/global/AdobeDocumentServices/lib/XDC/Customer/`
for each local SID with the ADS component enabled.

Default value: `$sap::deployed_xdc`

### <a name="sapmanagementdb2_queries"></a>`sap::management::db2_queries`

Implementation detail - creates sql query scheduling infrastructure

#### Parameters

The following parameters are available in the `sap::management::db2_queries` class:

* [`system_ids`](#system_ids)

##### <a name="system_ids"></a>`system_ids`

Data type: `Hash[Sap::SID, Sap::SIDConfigEntry]`

Replicated from sap::system_ids primarily to make the test code easier to
write. See the parent for further detail.

Default value: `$sap::system_ids`

### <a name="sapmanagementdotfiles"></a>`sap::management::dotfiles`

This is an internal class and should not be called directly.
If $bash_source_profile is set to true for the instance as a whole this class
will be enabled and will cause the .bash_profile of the sid_adm and db2 users
to source their '.profile'.

### <a name="sapmanagementetc_services"></a>`sap::management::etc_services`

This class is an implementation detail and should not be called directly.

#### Parameters

The following parameters are available in the `sap::management::etc_services` class:

* [`message_servers`](#message_servers)
* [`system_ids`](#system_ids)

##### <a name="message_servers"></a>`message_servers`

Data type: `Hash[Sap::SID, Sap::MessageServer]`

Sourced directly from the parent message_servers parameter on the SAP class.
See that for further detail.

##### <a name="system_ids"></a>`system_ids`

Data type: `Hash[Sap::SID, Sap::SIDConfigEntry]`

Sourced directly from the parent system_ids parameter on the SAP class. See
that for further detail.

### <a name="sapmanagementfirewall"></a>`sap::management::firewall`

This class is an implementation detail and should not be called directly.

#### Parameters

The following parameters are available in the `sap::management::firewall` class:

* [`system_ids`](#system_ids)
* [`service_map`](#service_map)
* [`services`](#services)

##### <a name="system_ids"></a>`system_ids`

Data type: `Hash[Sap::SID, Sap::SIDConfigEntry]`

Sourced directly from the parent system_ids parameter on the SAP class. See
that class for further detail.

##### <a name="service_map"></a>`service_map`

Data type: `Hash[Sap::InstClass, Array[Sap::FirewallServices]]`

Mapping from instance class types to the corresponding SAP firewall service
names. If a given instance class is detected on the server it will present
configure the corresponding firewall services.

##### <a name="services"></a>`services`

Data type: `Hash[Sap::FirewallServices, Sap::FirewallRule]`

Hash defining each firewall service entry

### <a name="sapmanagementpermissions"></a>`sap::management::permissions`

Implementation detail - corrects permissions as needed

#### Parameters

The following parameters are available in the `sap::management::permissions` class:

* [`system_ids`](#system_ids)

##### <a name="system_ids"></a>`system_ids`

Data type: `Hash[Sap::SID, Sap::SIDConfigEntry]`

Replicated from sap::system_ids primarily to make the test code easier to
write. See the parent for further detail.

Default value: `$sap::system_ids`

### <a name="sapmanagementprofiles"></a>`sap::management::profiles`

This class is an implementation detail and should not be called directly.
Currently only the DEFAULT.PFL file is managed by this class.

#### Parameters

The following parameters are available in the `sap::management::profiles` class:

* [`system_ids`](#system_ids)

##### <a name="system_ids"></a>`system_ids`

Data type: `Hash[Sap::SID, Sap::SIDConfigEntry]`

Sourced directly from the parent system_ids parameter on the SAP class. See
that class for further detail.

### <a name="sapparams"></a>`sap::params`

General parameter definitions for the SAP Netweaver environment are defined
within this module. Note that OS and Architecture specific variants are
actually pulled from defaults in Hiera.

#### Parameters

The following parameters are available in the `sap::params` class:

* [`config_redhat_release_conf`](#config_redhat_release_conf)
* [`service_uuidd`](#service_uuidd)
* [`service_scc`](#service_scc)
* [`service_saprouter`](#service_saprouter)
* [`config_saproutetab`](#config_saproutetab)
* [`config_saproutetab_template`](#config_saproutetab_template)
* [`config_saprouter_sysconfig`](#config_saprouter_sysconfig)
* [`config_saprouter_sysconfig_template`](#config_saprouter_sysconfig_template)
* [`config_sysctl`](#config_sysctl)
* [`config_limits`](#config_limits)
* [`config_default_mount_options`](#config_default_mount_options)

##### <a name="config_redhat_release_conf"></a>`config_redhat_release_conf`

Data type: `String`

This should point to the path of the desired redhat release config file.

Default value: ``undef``

##### <a name="service_uuidd"></a>`service_uuidd`

Data type: `String`

Name of the uuidd service. This is used to enable the daemon after the
installation completes.

Default value: ``undef``

##### <a name="service_scc"></a>`service_scc`

Data type: `String`

SAP Cloud Connector service name used for enabling the service.

Default value: ``undef``

##### <a name="service_saprouter"></a>`service_saprouter`

Data type: `String`

Name of the service corresponding to the SAP router process.

Default value: ``undef``

##### <a name="config_saproutetab"></a>`config_saproutetab`

Data type: `String`

File target for the sap router route-table should it be configure.

Default value: ``undef``

##### <a name="config_saproutetab_template"></a>`config_saproutetab_template`

Data type: `String`

Path within the module to the appropriate erb template to apply for the
saproutetab.

Default value: ``undef``

##### <a name="config_saprouter_sysconfig"></a>`config_saprouter_sysconfig`

Data type: `String`

Target /etc/sysconfig file for the saprouter service

Default value: ``undef``

##### <a name="config_saprouter_sysconfig_template"></a>`config_saprouter_sysconfig_template`

Data type: `String`

Embedded ruby template to be applied to the sysconfig file

Default value: ``undef``

##### <a name="config_sysctl"></a>`config_sysctl`

Data type: `Hash[
    String,
    Hash[
      String,
      Variant[
        String,
        Integer,
        Array[String],
        Hash[
          String,
          Variant[
            Hash[
              String,
              Variant[
                String,
                Integer,
                Array[
                  Variant[
                    String,
                    Hash[
                      String,
                      Variant[
                        String,
                        Integer,
                      ]
                    ],
                  ]
                ]
              ]
            ]
          ]
        ]
      ]
    ]
  ]`

Similar to config_limits this provides a configurable set of directives to
include in /etc/sysctl.d/ on a per component basis.

Default value: `{}`

##### <a name="config_limits"></a>`config_limits`

Data type: `Hash[
    String,
    Hash[
      String,
      Variant[
        String,
        Boolean,
        Array[String],
        Hash[
          String,
          Hash[
            String,
            Hash[
              String,
              Variant[
                String,
                Integer,
              ]
            ]
          ]
        ]
      ]
    ]
  ]`

A deeply nested hash containing the target /etc/security/limits.d/
entries for a given system. This is currently defined for baseline sap
instances and db2. The hash has the following structure

```puppet
'component-name' => {
  path           => '/etc/security/limits.d', # this should be left alone - limits directory
  sequence       => '00', # string ID prepended to the limits file. e.g.
                          # /etc/security/limits.d/00-sap-base.conf
  per_sid        => true, # Indicates whether the domain has a _sid_
                          # substring that will need to be substituted with the actual system ID.
  header_comment => [ # An array of comment lines which will appear at the
                      #start of the file
    'Comment line 1',
    'Comment line 2',
  ],
  entries => { # Hash containing the actual
  }
```

Default value: `{}`

##### <a name="config_default_mount_options"></a>`config_default_mount_options`

Data type: `Hash[
    String,
    Hash[
      String,
      String,
    ]
  ]`

Default options to provide to provide for the mount operation in the case
that they are not overridden locally. See
https://puppet.com/docs/puppet/latest/types/mount.html#mount-attribute-options
for details. Note that this is a hash of hashes with an entry for each
supported resource type.

Default value: `{}`

### <a name="sapservice"></a>`sap::service`

Configures OS level services required for SAP Netweaver instances

### <a name="sapservicecloudconnector"></a>`sap::service::cloudconnector`

SAP Cloud Connector service configuration

### <a name="sapservicerouter"></a>`sap::service::router`

Configures the SAP Router service

## Defined types

### <a name="sapclusteras"></a>`sap::cluster::as`

The sap::cluster::as class.

#### Examples

##### Asymmetric 4-node cluster defining the AS resource for each app node

```puppet
sap::cluster::as { 'PR0_As':
  sid          => 'PR0',
  cib_name     => 'sap',
  nodes    => {
    'pr0-app00.example.org' => ['app-standard'],
    'pr0-app01.example.org' => ['app-standard'],
    'pr0-dba.example.org'   => ['db-standard'],
    'pr0-dbb.example.org'   => ['db-standard'],
  },
  as_instances => [
    {
      'type'   => 'DVEBMGS',
      'number' => '00',
      'host'   => 'pr0-app00',
    },
    {
      'type'   => 'D',
      'number' => '00',
      'host'   => 'pr0-app01',
    },
  ],
}
```

#### Parameters

The following parameters are available in the `sap::cluster::as` defined type:

* [`cib_name`](#cib_name)
* [`nodes`](#nodes)
* [`as_instances`](#as_instances)
* [`db_mode`](#db_mode)
* [`sid`](#sid)

##### <a name="cib_name"></a>`cib_name`

Data type: `String`

Name of the temporary Cluster Information Base to use during the building
and commiting of the rule changes incorporated in this module.

##### <a name="nodes"></a>`nodes`

Data type: `Sap::ClusterNodeRoles`

Mapping containing each a list of relevant roles for each node in the
cluster.

##### <a name="as_instances"></a>`as_instances`

Data type: `Array[Sap::InstData]`

Each entry in this array corresponds to a unique standalone instance which
is linked to a home host. Entries must have the following three values:
* `type' - 'Name' of the instance type. For example 'D', 'DVEBMGS', or 'J'
* `number' - two digit code assigned to the instance.
* `host' - short hostname used to install the instance.

##### <a name="db_mode"></a>`db_mode`

Data type: `Sap::ClusterDbMode`

Specifies the type of clustering used for the underlying database. This
information is use to construct the start-after relationship on the core
database PCS resource.

##### <a name="sid"></a>`sid`

Data type: `Sap::SID`



### <a name="sapclusteras_inst"></a>`sap::cluster::as_inst`

This defined type should be used by the sap::cluster class and not called
directly externally.

#### Examples

##### Definition for an instance on pr0-app01

```puppet
sap::cluster::as_inst { 'as_inst_PR0_0':
  sid            => 'PR0',
  cib_name       => 'sap',
  scs_primitive  => 'ms_prm_PR0_SCS',
  db_mode        => 'log_shipping',
  profile_dir    => '/sapmnt/PR0/profile',
  relevant_nodes => ['pr0-app00.example.org', 'pr0-app01.example.org'],
  type           => 'DVEBMGS',
  number         => '00',
  host           => 'pr0-app01',
}
```

#### Parameters

The following parameters are available in the `sap::cluster::as_inst` defined type:

* [`sid`](#sid)
* [`cib_name`](#cib_name)
* [`db_mode`](#db_mode)
* [`profile_dir`](#profile_dir)
* [`relevant_nodes`](#relevant_nodes)
* [`type`](#type)
* [`number`](#number)
* [`host`](#host)
* [`legacy_start_profile`](#legacy_start_profile)

##### <a name="sid"></a>`sid`

Data type: `Sap::SID`

System ID this instance is associated with

##### <a name="cib_name"></a>`cib_name`

Data type: `String`

Name of the temporary Cluster Information Base to use during the building
and commiting of the rule changes incorporated in this module.

##### <a name="db_mode"></a>`db_mode`

Data type: `Sap::ClusterDbMode`

Pass-through parameter from the parent sap::cluster::as type. See that
description for details.

##### <a name="profile_dir"></a>`profile_dir`

Data type: `Stdlib::AbsolutePath`

Location storing the profile files for each instance. Typically this path is
defined as `/sapmnt/${SID}/profile`. The full start profile is constructed
using the instance specific data provided in other parameters.

##### <a name="relevant_nodes"></a>`relevant_nodes`

Data type: `Array[Stdlib::Host]`

A filtered list of nodes which are relevant to the AS instance described by
this instance. One entry in this must match the `host` parameter defined
below or the declaration will fail.

##### <a name="type"></a>`type`

Data type: `Sap::InstType`

'Name' of the instance type. For example 'D', 'DVEBMGS', or 'J'.

##### <a name="number"></a>`number`

Data type: `Sap::InstNumber`

Two digit code assigned to the instance.

##### <a name="host"></a>`host`

Data type: `Stdlib::Host`

Short hostname used when the instance was installed.

##### <a name="legacy_start_profile"></a>`legacy_start_profile`

Data type: `Boolean`

Indicates whether this instance uses the separate START_* profiles as was
common on older releases of SAP Netweaver prior to 7.10. More recent builds
include the START details within the standard profile. For further detail
see https://launchpad.support.sap.com/#/notes/1751381.

Default value: ``false``

### <a name="sapclusterdb2"></a>`sap::cluster::db2`

This defined type should be used by the sap::cluster class and not called
directly externally.

#### Examples

##### Asymmetric 4-node cluster with DB2 HADR on two of the nodes

```puppet
sap::cluster::scsers { 'PR0_Db2':
  sid          => 'PR0',
  cib_name     => 'sap',
  nodes        => {
    'pr0-app00.example.org' => ['app-standard'],
    'pr0-app01.example.org' => ['app-standard'],
    'pr0-dba.example.org'   => ['db-standard'],
    'pr0-dbb.example.org'   => ['db-standard'],
  },
  db_ip        => '10.0.128.15',
  mode         => 'log_shipping',
}
```

#### Parameters

The following parameters are available in the `sap::cluster::db2` defined type:

* [`sid`](#sid)
* [`cib_name`](#cib_name)
* [`nodes`](#nodes)
* [`db_ip`](#db_ip)
* [`mode`](#mode)
* [`volumes`](#volumes)
* [`filesystems`](#filesystems)
* [`hadr_peer_window`](#hadr_peer_window)

##### <a name="sid"></a>`sid`

Data type: `Sap::SID`

System ID this instance is associated with

##### <a name="cib_name"></a>`cib_name`

Data type: `String`

Name of the temporary Cluster Information Base to use during the building
and commiting of the rule changes incorporated in this module.

##### <a name="nodes"></a>`nodes`

Data type: `Sap::ClusterNodeRoles`

Mapping containing each a list of relevant roles for each node in the
cluster.

##### <a name="db_ip"></a>`db_ip`

Data type: `Stdlib::IP::Address::Nosubnet`

Service IP address which moves between nodes and represents the 'master'
of the cluster. Clients will connect to this instead of
the local IPs of the machines.

##### <a name="mode"></a>`mode`

Data type: `Sap::ClusterDbMode`

Indicates the type of HA cluster. There are currently two supported modes
* `log_shipping` - Shared nothing approach which utilizes log shipping
    to keep one or more replicas synchronized with the current primary.
* `shared_storage` - Approach which moves disks containing the
    database instance between one or more nodes.

##### <a name="volumes"></a>`volumes`

Data type: `Array[Sap::ClusterLvm]`

Array containing a series of hashes which describe the LVM cluster resources
needed for the primary node of this db2 cluster to function.

##### <a name="filesystems"></a>`filesystems`

Data type: `Array[Sap::ClusterFilesystem]`

Array containing a series of hashes which describe the Filesystem cluster
resources needed on the primary node. The parent LVM volume for each file
system must be included in the `volume` resource.

##### <a name="hadr_peer_window"></a>`hadr_peer_window`

Data type: `Integer`

Maximum number of seconds the standby nodes can be in an HADR peer topology.
This is used to construct 'sane' monitoring intervals for the log-shipping
based DB2 model.

Default value: `300`

### <a name="sapclusterfence_lpar"></a>`sap::cluster::fence_lpar`

This defined type should be used by the sap::cluster class and not called
directly externally.

#### Examples

##### Asymmetric 4-node cluster defining the AS resource for each app node

```puppet
sap::cluster::as { 'fence_lpar_0':
  cib_name                  => 'sap',
  ipaddr                    => 'hmc0.example.org',
  login                     => 'service-fence',
  managed                   => 'host0',
  identity_file             => '/root/.ssh/hmc_secret.rsa',
  pcmk_delay_max            => 15,
  pcmk_host_map             => {
    'pr0-app00.example.org' => 'pr0-app00',
    'pr0-app01.example.org' => 'pr0-app01',
    'pr0-dba.example.org'   => 'pr0-dba',
    'pr0-dbb.example.org'   => 'pr0-dbb',
  },
}
```

#### Parameters

The following parameters are available in the `sap::cluster::fence_lpar` defined type:

* [`cib_name`](#cib_name)
* [`ipaddr`](#ipaddr)
* [`login`](#login)
* [`managed`](#managed)
* [`identity_file`](#identity_file)
* [`pcmk_delay_max`](#pcmk_delay_max)
* [`pcmk_host_map`](#pcmk_host_map)
* [`node_list`](#node_list)

##### <a name="cib_name"></a>`cib_name`

Data type: `String`

Name of the temporary Cluster Information Base to use during the building
and commiting of the rule changes incorporated in this module.

##### <a name="ipaddr"></a>`ipaddr`

Data type: `Stdlib::Host`

IP address or hostname of the fencing HMC

##### <a name="login"></a>`login`

Data type: `String`

Login username for the HMC

##### <a name="managed"></a>`managed`

Data type: `Stdlib::Host`

Name of the CEC or managed system which hosts the target LPAR(s)

##### <a name="identity_file"></a>`identity_file`

Data type: `Stdlib::AbsolutePath`

Private key file used to connect to the HMC

##### <a name="pcmk_delay_max"></a>`pcmk_delay_max`

Data type: `Integer`

a random maximum delay for stonith actions to prevent double fencing

##### <a name="pcmk_host_map"></a>`pcmk_host_map`

Data type: `Sap::ClusterFenceHostMap`

One to one mapping from trusted node host names to the LPAR names in the
HMC. An individual stonith resource will be created for each entry in this
list and will be allowed to run on any node other than the node it is
responsible for fencing.

##### <a name="node_list"></a>`node_list`

Data type: `Array[Stdlib::Host]`

Full list of nodes in the cluster. This is used to co-locate the resource on
any node other than the home node.

### <a name="sapclusterfilesystem"></a>`sap::cluster::filesystem`

This defined type should be used by the sap::cluster class and not called
directly externally.

#### Examples

##### Definition for a shared commvault volume

```puppet
sap::cluster::as_inst { 'db2pr0_lvm_0':
  sid            => 'PR0',
  cib_name       => 'sap',
  volume_group   => 'db2cv',
  logical_volume => 'db2cvarch,
  fstype         => 'xfs',
  directory      => '/db2/PR0/commvault/archive',
}
```

#### Parameters

The following parameters are available in the `sap::cluster::filesystem` defined type:

* [`sid`](#sid)
* [`cib_name`](#cib_name)
* [`lvm_primitives`](#lvm_primitives)
* [`volume_group`](#volume_group)
* [`logical_volume`](#logical_volume)
* [`fstype`](#fstype)
* [`directory`](#directory)
* [`device`](#device)

##### <a name="sid"></a>`sid`

Data type: `Sap::SID`

System ID this instance is associated with

##### <a name="cib_name"></a>`cib_name`

Data type: `String`

Name of the temporary Cluster Information Base to use during the building
and commiting of the rule changes incorporated in this module.

##### <a name="lvm_primitives"></a>`lvm_primitives`

Data type: `Array[String]`

Array containing the list of lvm primitives managed by the parent type. It
must contain an entry which matches the expected primitive described by the
volume_group parameter for this filesystem.

##### <a name="volume_group"></a>`volume_group`

Data type: `Sap::VgLvName`

Name of the LVM volume group to be administered. Used to generate the
device parameter.

##### <a name="logical_volume"></a>`logical_volume`

Data type: `Sap::VgLvName`

Name of the logical volume containing this filesystem. This is used to
generate the device parameter.

##### <a name="fstype"></a>`fstype`

Data type: `Sap::FsType`

Format of the filesystem.

##### <a name="directory"></a>`directory`

Data type: `Stdlib::AbsolutePath`

Mountpoint where this filesystem is attached.

##### <a name="device"></a>`device`

Data type: `Stdlib::AbsolutePath`

Path to the underlying block device on which this filesystem resides.
Typically this is constructed with the volume group name and the logical
volume name.

Default value: `"/dev/${volume_group}/${logical_volume}"`

### <a name="sapclusterlvm"></a>`sap::cluster::lvm`

This defined type should be used by the sap::cluster class and not called
directly externally.

#### Examples

##### Definition for a shared commvault volume

```puppet
sap::cluster::as_inst { 'db2pr0_lvm_0':
  sid                => 'PR0',
  cib_name           => 'sap',
  volgrpname         => 'db2cv',
  exclusive          => true,
  partial_activation => false,
}
```

#### Parameters

The following parameters are available in the `sap::cluster::lvm` defined type:

* [`sid`](#sid)
* [`cib_name`](#cib_name)
* [`volgrpname`](#volgrpname)
* [`exclusive`](#exclusive)
* [`partial_activation`](#partial_activation)

##### <a name="sid"></a>`sid`

Data type: `Sap::SID`

System ID this instance is associated with

##### <a name="cib_name"></a>`cib_name`

Data type: `String`

Name of the temporary Cluster Information Base to use during the building
and commiting of the rule changes incorporated in this module.

##### <a name="volgrpname"></a>`volgrpname`

Data type: `Sap::VgLvName`

Name of the LVM volume group to be administered.

##### <a name="exclusive"></a>`exclusive`

Data type: `Boolean`

When enabled the specified volume group will be exclusively activated.
see `pcs resource describe LVM` for details.

Default value: ``true``

##### <a name="partial_activation"></a>`partial_activation`

Data type: `Boolean`

When enabled corosync/pcs will attempt to activate the volume even if some
disks are missing. This can be extremely useful when LVM mirroring is
configured.

Default value: ``false``

### <a name="sapclusterscsers"></a>`sap::cluster::scsers`

This defined type should be used by the sap::cluster class and not called
directly externally.

#### Examples

##### Asymmetric 4-node cluster with two app relevant nodes

```puppet
sap::cluster::scsers { 'PR0_ScsErs':
  sid       => 'PR0',
  cib_name  => 'sap',
  nodes     => {
    'pr0-app00.example.org' => ['app-scs'],
    'pr0-app01.example.org' => ['app-scs'],
    'pr0-dba.example.org'   => ['db-standard'],
    'pr0-dbb.example.org'   => ['db-standard'],
  },
  ers_ip    => '10.0.0.11'
  ers_inst  => {
    'type'   => 'ERS',
    'number' => '02',
    'host'   => 'pr0-ers',
  },
  scs_ip    => '10.0.0.10'
  scs_inst  => {
    'type'   => 'ASCS',
    'number' => '01',
    'host'   => 'pr0-cs',
  },
}
```

#### Parameters

The following parameters are available in the `sap::cluster::scsers` defined type:

* [`sid`](#sid)
* [`cib_name`](#cib_name)
* [`nodes`](#nodes)
* [`ers_ip`](#ers_ip)
* [`ers_inst`](#ers_inst)
* [`scs_ip`](#scs_ip)
* [`scs_inst`](#scs_inst)

##### <a name="sid"></a>`sid`

Data type: `Sap::SID`

System ID this instance is associated with

##### <a name="cib_name"></a>`cib_name`

Data type: `String`

Name of the temporary Cluster Information Base to use during the building
and commiting of the rule changes incorporated in this module.

##### <a name="nodes"></a>`nodes`

Data type: `Sap::ClusterNodeRoles`

Mapping containing each a list of relevant roles for each node in the
cluster.

##### <a name="ers_ip"></a>`ers_ip`

Data type: `Stdlib::IP::Address::Nosubnet`

IP address assigned to the ERS virtual host.

##### <a name="ers_inst"></a>`ers_inst`

Data type: `Sap::InstData`

Structure providing details for the ERS instance. There are up to four
components. Note that the first three are mandatory, however, the fourth is
optional.
* `type` - Type of ERS instance. This should pretty much always be 'ERS'
* `number` - Two digit instance ID
* `host` - short-name of the ERS virtual host.
* `legacy_start_profile` - Boolean value indicating whether this is a sap
   instance with 'START_' profiles separate from the instance ala-pre NW 7.10

##### <a name="scs_ip"></a>`scs_ip`

Data type: `Stdlib::IP::Address::Nosubnet`

IP address assigned to the SCS virtual host.

##### <a name="scs_inst"></a>`scs_inst`

Data type: `Sap::InstData`

Structure providing details for the SCS instance. There are up to four
components. Note that the first three are mandatory, however, the fourth is
optional.
* `type` - Type of SCS instance. Could be either SCS or ASCS
* `number` - Two digit instance ID
* `host` - short-name of the SCS virtual host.
* `legacy_start_profile` - Boolean value indicating whether this is a sap
   instance with 'START_' profiles separate from the instance ala-pre NW 7.10

### <a name="sapconfigmount_point"></a>`sap::config::mount_point`

This type is an implementation detail and should not be used directly!

#### Parameters

The following parameters are available in the `sap::config::mount_point` defined type:

* [`mount_path`](#mount_path)
* [`file_params`](#file_params)
* [`count`](#count)
* [`mount_params`](#mount_params)
* [`required_files`](#required_files)
* [`mount_defaults`](#mount_defaults)
* [`per_sid`](#per_sid)
* [`pattern_map`](#pattern_map)
* [`pattern_values`](#pattern_values)

##### <a name="mount_path"></a>`mount_path`

Data type: `String`

The path to the mountpoint. Note that this is a namevar.

Default value: `$name`

##### <a name="file_params"></a>`file_params`

Data type: `Hash[String, String]`

Any valid parameter which could be provided to a File resource type can be
included in this hash. Note that ensure is always set to `directory`.

##### <a name="count"></a>`count`

Data type: `Optional[Integer]`

Indicates that this path should be created $count times. The path must
contain the string `_N_` which will be substituted with the count.
For example `/db2/ECP/sapdata_N_`

Default value: ``undef``

##### <a name="mount_params"></a>`mount_params`

Data type: `Hash`

TODO - fill out this section

Default value: `{}`

##### <a name="required_files"></a>`required_files`

Data type: `Array[String]`

Ordered list of files which this mount point should require.

Default value: `[]`

##### <a name="mount_defaults"></a>`mount_defaults`

Data type: `Hash`

Hash of hashes containing mount-type specific defaults. Currently this is
used exclusively to provide default parameters to the NFSv4 client mounts
for SAP related connections.

Default value: `{}`

##### <a name="per_sid"></a>`per_sid`

Data type: `Sap::MountPointSIDMode`

Controls the validation behavior for the module via three possible states
`true` - the path is SID specific and an instance can be created for each
`false` - only one occurance of the path can exist on a system
`user` - the entry has SID specific attributes but the path is unique

##### <a name="pattern_map"></a>`pattern_map`

Data type: `Hash[String, String]`

Hash mapping potential `pattern_values` to the regular expression patterns
which they should replace. The value retrieved by the same key in
`pattern_values` will be substituted into the mount_path, owner, and group
of the target file wherever the regular expression matches. Note that the
patterns should be provided as strings, however, they will be converted to
regexp for use via `Regexp($value)`

##### <a name="pattern_values"></a>`pattern_values`

Data type: `Hash[String, String]`

Hash mapping substitution targets with their corresponding values for this
mountpoint instance. Typically substitutions are SID specific and are
specified globally via the `$sap::system_ids['pattern_map'] parameter.

### <a name="sapmanagementdb2_query_schedule"></a>`sap::management::db2_query_schedule`

Implementation detail - a query file & associate cron schedule

#### Parameters

The following parameters are available in the `sap::management::db2_query_schedule` defined type:

* [`sid`](#sid)
* [`query_file_name`](#query_file_name)
* [`query_content`](#query_content)
* [`cron_entry`](#cron_entry)
* [`query_directory`](#query_directory)
* [`schema`](#schema)
* [`output_target`](#output_target)
* [`output_group`](#output_group)
* [`monitor_alias`](#monitor_alias)

##### <a name="sid"></a>`sid`

Data type: `Sap::SID`

System ID for this query

##### <a name="query_file_name"></a>`query_file_name`

Data type: `String[1]`

Basename of the file defining this query. Note that this file must be a bash
script which implements the function `execute_query`.

##### <a name="query_content"></a>`query_content`

Data type: `Sap::SourceURL`

URL to the content of the query file which will be deployed.

##### <a name="cron_entry"></a>`cron_entry`

Data type: `Sap::ScheduleEntry`

Cron schedule entry defining the frequency at which this query will run

##### <a name="query_directory"></a>`query_directory`

Data type: `Stdlib::AbsolutePath`

Location in which this (and all other) queries will be installed. Note that
this is set by the sap::management::db2_queries class

##### <a name="schema"></a>`schema`

Data type: `Optional[Sap::Db2Schema]`

In some cases the schema name of a given system does not match the SID. This
is typically the case for redirected restores in QA environments. If this
field is specified it overrides the standard "SAP${sid}" schema name.

Default value: ``undef``

##### <a name="output_target"></a>`output_target`

Data type: `Optional[Stdlib::AbsolutePath]`

Specifies the target result file of the scheduled query. Typically this only
applies to queries which perform some form of export statement.

Default value: ``undef``

##### <a name="output_group"></a>`output_group`

Data type: `Optional[String[1]]`

Operating system group which will be set as the owning group of the
output_target file after it is generated.

Default value: ``undef``

##### <a name="monitor_alias"></a>`monitor_alias`

Data type: `Optional[Stdlib::Host]`

IPv4 address or DNS name representing a service IP that may be present on
one or more nodes. In the case of an HA/DR cluster execution of this query
will be restricted to the node which currently has this address.

Default value: ``undef``

## Functions

### <a name="multiregsubst"></a>`multiregsubst`

Type: Ruby 4.x API

Performs a series of substitutions on the same target string

#### Examples

##### Two patterns

```puppet
$pattern_map = {/_sid_/ => 'scp', /_SID_/ => 'SCP'}
$target = '/db2/_SID_/db2_sid_'
$result = multiregsubst($target, $pattern_map)
# The output string will be equivalent to the following
# $result = '/db2/SCP/db2scp'
```

#### `multiregsubst(String $target, Hash[Regexp, String] $substmap)`

Performs a series of substitutions on the same target string

Returns: `String` Returns target after all substitutions

##### Examples

###### Two patterns

```puppet
$pattern_map = {/_sid_/ => 'scp', /_SID_/ => 'SCP'}
$target = '/db2/_SID_/db2_sid_'
$result = multiregsubst($target, $pattern_map)
# The output string will be equivalent to the following
# $result = '/db2/SCP/db2scp'
```

##### `target`

Data type: `String`

The string to be substituted

##### `substmap`

Data type: `Hash[Regexp, String]`

Hash mapping regexp to target value

### <a name="sapclusterfilter_nodes"></a>`sap::cluster::filter_nodes`

Type: Puppet Language

Simple function to retrieve only the nodes with a desired role

#### `sap::cluster::filter_nodes(Sap::NodeRole $target_role, Sap::ClusterNodeRoles $node_role_hash)`

The sap::cluster::filter_nodes function.

Returns: `Array[Stdlib::Fqdn]` Array of node names which have the target_role role.

##### `node_role_hash`

Data type: `Sap::ClusterNodeRoles`

Mapping from node names to a list of roles which that node can perform.

##### `target_role`

Data type: `Sap::NodeRole`

Nodes with this role will be returned in the resulting list

### <a name="sapsysctl_calculated_values"></a>`sap::sysctl_calculated_values`

Type: Puppet Language

Converts the provided data hash into the corresponding value. This class
performs recursive calls to construct a single value when provided with a
compound series of values.

#### `sap::sysctl_calculated_values(Array[Hash] $data, Integer $index)`

Converts the provided data hash into the corresponding value. This class
performs recursive calls to construct a single value when provided with a
compound series of values.

Returns: `String` The value computed by the calculation process.

##### `data`

Data type: `Array[Hash]`

Array of value hashes. The function will first attempt to call itself for
the next entry in the array and then prepend it's own calculation to
assemble the resulting string.

##### `index`

Data type: `Integer`

Current position in the provided array. The function will call itself for
index + 1 unless the current index is the last element in the array.

### <a name="sapsysctl_line"></a>`sap::sysctl_line`

Type: Puppet Language

This function converts a given entry from a module specific hash provided to
sap::params::config_sysctl into a line of the form <directive> = <value>.
There are three supported line types for this component:
1. `value` - Simply sets the value equal to the entry name
2. `calculated` - Computes the value via the following formula `base * multiplier / divisor`
3. `compound` - Performs a recursive call converting multiple value/calculated sub entries into a single space separated value.

#### `sap::sysctl_line(String $param, Hash[
    String,
    Variant[
      String,
      Integer,
      Array[
        Variant[
          String,
          Hash[
            String,
            Variant[
              Integer,
              String,
            ]
          ]
        ]
      ]
    ]
  ] $data)`

This function converts a given entry from a module specific hash provided to
sap::params::config_sysctl into a line of the form <directive> = <value>.
There are three supported line types for this component:
1. `value` - Simply sets the value equal to the entry name
2. `calculated` - Computes the value via the following formula `base * multiplier / divisor`
3. `compound` - Performs a recursive call converting multiple value/calculated sub entries into a single space separated value.

Returns: `String` A string of the form <directive> = <value>. For example:
`kernel.sem = 1250 256000 100 8192`

##### `param`

Data type: `String`

The name of the kernel parameter this corresponds to. E.g. `kernel.sem` is
the parameter for Linux kernel Semaphore parameters.

##### `data`

Data type: `Hash[
    String,
    Variant[
      String,
      Integer,
      Array[
        Variant[
          String,
          Hash[
            String,
            Variant[
              Integer,
              String,
            ]
          ]
        ]
      ]
    ]
  ]`

Detailed structure containing the components needed to produce the value for
the given parameter.

## Data types

### <a name="sapabaplang"></a>`Sap::ABAPLang`

The Sap::ABAPLang data type.

Alias of

```puppet
Pattern[/\A[A-Z]{2}\z/]
```

### <a name="sapabaplogingroup"></a>`Sap::ABAPLoginGroup`

The Sap::ABAPLoginGroup data type.

Alias of

```puppet
Pattern[/\A[A-Z_0-9]{1,20}\z/]
```

### <a name="sapabappassword"></a>`Sap::ABAPPassword`

Ideally: type Sap::ABAPPassword = Sensitive[Pattern[/\A[[:graph:]]{1,40}\z/]]

Alias of

```puppet
Sensitive[String]
```

### <a name="sapabapuser"></a>`Sap::ABAPUser`

The Sap::ABAPUser data type.

Alias of

```puppet
Pattern[/\A[[:upper:][:digit:]_*-]{1,12}\z/]
```

### <a name="sapabapclient"></a>`Sap::AbapClient`

The Sap::AbapClient data type.

Alias of

```puppet
Pattern[/\A[0-9]{3}\z/]
```

### <a name="sapbackenddatabase"></a>`Sap::BackendDatabase`

Constrained list of backend databases SAP might connect to

Alias of

```puppet
Enum['db6', 'liveCache', 'maxdb']
```

### <a name="sapclusteras"></a>`Sap::ClusterAs`

Used to model linux high availability rules for one or more application server
instances. Note that this does not support the 'primary application server'
model.

Alias of

```puppet
Hash[Enum[
    'as_instances',    # Instance details for each AS
  ], Variant[
    Array[Sap::InstData],
  ]]
```

### <a name="sapclustercomponent"></a>`Sap::ClusterComponent`

Higher level model for linux high-availability rules which captures the type
and any subcomponents of each component

Alias of

```puppet
Hash[Variant[Enum['type','data']], Variant[
    Sap::ClusterComponentType,
    Sap::ClusterScsErs,
    Sap::ClusterAs,
    Sap::ClusterLvm,
    Sap::ClusterDb2,
  ]]
```

### <a name="sapclustercomponenttype"></a>`Sap::ClusterComponentType`

Constrains the list of valid resource types

Alias of

```puppet
Enum['As', 'ScsErs', 'Db2', 'Lvm']
```

### <a name="sapclusterdb2"></a>`Sap::ClusterDb2`

Used to model linux high availability rules for a HADR based DB2 cluster
Note that this assumes a single SID per instance. (SAP typically assumes this
too!)

Alias of

```puppet
Hash[Enum[
    'db_ip',            # Service IP corresponding to the database cluster
    'mode',             # Type of DB2 cluster to configure.
    'hadr_peer_window', # Peer window in seconds used by HADR
    'volumes',          # Array of LVM volume group resources colocated w/ the master
    'filesystems',      # Array of filesystem resources colocated w/ the master
  ], Variant[
    Stdlib::IP::Address::Nosubnet,
    Sap::ClusterDbMode,
    Integer,
    Array[Sap::ClusterLvm],
    Array[Sap::ClusterFilesystem],
  ]]
```

### <a name="sapclusterdbmode"></a>`Sap::ClusterDbMode`

The Sap::ClusterDbMode data type.

Alias of

```puppet
Enum['log_shipping', 'shared_storage']
```

### <a name="sapclusterfence"></a>`Sap::ClusterFence`

The Sap::ClusterFence data type.

Alias of

```puppet
Hash[Enum[
    'type',
    'data',
  ], Variant[
    Sap::ClusterFenceType,
    Sap::ClusterFenceData,
  ]]
```

### <a name="sapclusterfencedata"></a>`Sap::ClusterFenceData`

The Sap::ClusterFenceData data type.

Alias of

```puppet
Variant[Sap::ClusterFenceDataLpar]
```

### <a name="sapclusterfencedatalpar"></a>`Sap::ClusterFenceDataLpar`

The Sap::ClusterFenceDataLpar data type.

Alias of

```puppet
Hash[Enum[
    'identity_file',
    'ipaddr',
    'login',
    'managed',
    'pcmk_delay_max',
    'pcmk_host_map',
  ], Variant[
    Stdlib::AbsolutePath,
    Stdlib::Host,
    String,
    Integer,
    Sap::ClusterFenceHostMap,
  ]]
```

### <a name="sapclusterfencehostmap"></a>`Sap::ClusterFenceHostMap`

The Sap::ClusterFenceHostMap data type.

Alias of

```puppet
Hash[Stdlib::Host, String]
```

### <a name="sapclusterfencetype"></a>`Sap::ClusterFenceType`

The Sap::ClusterFenceType data type.

Alias of

```puppet
Enum['fence_lpar']
```

### <a name="sapclusterfilesystem"></a>`Sap::ClusterFilesystem`

The Sap::ClusterFilesystem data type.

Alias of

```puppet
Hash[Enum[
    'volume_group',   # Name of the LVM volume group backing this fs
    'logical_volume', # Name of the LVM logical volume containing this fs
    'directory',      # Mountpoint for this fs
    'fstype',         # Filesystem type used at this mount point
  ], Variant[
    Sap::VgLvName,
    Stdlib::AbsolutePath,
    Sap::FsType,
  ]]
```

### <a name="sapclusterlvm"></a>`Sap::ClusterLvm`

Parameters for an LVM volume

Alias of

```puppet
Hash[Enum[
    'volgrpname',         # Volume group to manage
    'exclusive',          # If set the volume will be activated on only one node
    'partial_activation', # When set, the volume group will activate even if some of the physical devices are missing
  ], Variant[
    Sap::VgLvName,
    Boolean,
  ]]
```

### <a name="sapclusternoderoles"></a>`Sap::ClusterNodeRoles`

Mapping from each node to an array listing all of the roles this node fulfills
within this cluster. For example
{
  'pr0-ap00' => [ 'app-standard', 'db-standard'],
  'pr0-ap01' => [ 'app-standard', 'db-standard'],
}

Alias of

```puppet
Hash[Stdlib::Fqdn, Array[Sap::NodeRole]]
```

### <a name="sapclusterscsers"></a>`Sap::ClusterScsErs`

Used to model linux high availability rules for an (A)SCS/ERS pair

Alias of

```puppet
Hash[Enum[
    'ers_ip',     # Service IP corresponding to the ERS
    'ers_inst',   # Instance details for the ERS
    'scs_ip',     # Service IP corresponding to the (A)SCS
    'scs_inst',   # Instance details for the (A)SCS
  ], Variant[
    Stdlib::IP::Address::Nosubnet,
    Sap::InstData,
  ]]
```

### <a name="sapdb2queryschedule"></a>`Sap::Db2QuerySchedule`

The Sap::Db2QuerySchedule data type.

Alias of

```puppet
Hash[Enum[
    'query_file_name',
    'query_content',
    'schema',
    'cron_entry',
    'output_target',
    'output_group',
    'monitor_alias',
  ], Variant[
    String,
    Sap::SourceURL,
    Sap::ScheduleEntry,
    Stdlib::AbsolutePath,
    Stdlib::Host,
  ]]
```

### <a name="sapdb2schema"></a>`Sap::Db2Schema`

The Sap::Db2Schema data type.

Alias of

```puppet
Pattern[/\ASAP[A-Z0-9]{3}\z/]
```

### <a name="sapdbhamode"></a>`Sap::DbHaMode`

The Sap::DbHaMode data type.

Alias of

```puppet
Enum['none', 'hadr']
```

### <a name="sapfirewallrule"></a>`Sap::FirewallRule`

The Sap::FirewallRule data type.

Alias of

```puppet
Hash[Enum[
    'desc',
    'ports',
  ], Variant[
    String,
    Array[Sap::FirewallRulePort],
  ]]
```

### <a name="sapfirewallruleport"></a>`Sap::FirewallRulePort`

The Sap::FirewallRulePort data type.

Alias of

```puppet
Hash[Enum[
    'port_pattern',
    'port',
    'protocol',
  ], Variant[
    Enum['tcp', 'udp'],
    String,
  ]]
```

### <a name="sapfirewallservices"></a>`Sap::FirewallServices`

The Sap::FirewallServices data type.

Alias of

```puppet
Enum['dispatcher', 'enqueue', 'gateway', 'igs', 'java_as', 'livecache', 'message_server', 'sapctrl', 'webdisp', 'web_icm']
```

### <a name="sapfstype"></a>`Sap::FsType`

Supported filesystems

Alias of

```puppet
Enum['xfs', 'ext4']
```

### <a name="saphostruleentry"></a>`Sap::HostRuleEntry`

The Sap::HostRuleEntry data type.

Alias of

```puppet
Variant[Stdlib::IP::Address::V4::Nosubnet, Stdlib::Fqdn, Pattern[/\A[*]\z/], Pattern[/\A([0-9]|[1-9][0-9]|1[0-9]{2}|2[0-4][0-9]|25[0-5])(\.([0-9]|[1-9][0-9]|1[0-9]{2}|2[0-4][0-9]|25[0-5])){2}(\.[*]){1}\z/], Pattern[/\A([0-9]|[1-9][0-9]|1[0-9]{2}|2[0-4][0-9]|25[0-5])(\.([0-9]|[1-9][0-9]|1[0-9]{2}|2[0-4][0-9]|25[0-5])){1}(\.[*]){2}\z/], Pattern[/\A([0-9]|[1-9][0-9]|1[0-9]{2}|2[0-4][0-9]|25[0-5])(\.[*]){3}\z/], Pattern[/\A[*](\.([a-zA-Z0-9]|[a-zA-Z0-9][a-zA-Z0-9\-]*[a-zA-Z0-9]))*\z/]]
```

### <a name="sapinstclass"></a>`Sap::InstClass`

The Sap::InstClass data type.

Alias of

```puppet
Variant[Sap::InstClassSap, Sap::InstClassDb, Sap::InstClassAux]
```

### <a name="sapinstclassaux"></a>`Sap::InstClassAux`

The Sap::InstClassAux data type.

Alias of

```puppet
Enum['scmopt', 'db-livecache-client']
```

### <a name="sapinstclassdb"></a>`Sap::InstClassDb`

The Sap::InstClassDb data type.

Alias of

```puppet
Enum['db-db2', 'db-livecache']
```

### <a name="sapinstclasssap"></a>`Sap::InstClassSap`

The Sap::InstClassSap data type.

Alias of

```puppet
Variant[Sap::InstClassSapApp, Sap::InstClassSapCs, Enum[
    'webdisp',
    'gateway',
    'ers',
  ]]
```

### <a name="sapinstclasssapapp"></a>`Sap::InstClassSapApp`

The Sap::InstClassSapApp data type.

Alias of

```puppet
Enum['as-dual', 'as-abap', 'as-java']
```

### <a name="sapinstclasssapcs"></a>`Sap::InstClassSapCs`

The Sap::InstClassSapCs data type.

Alias of

```puppet
Enum['cs-abap', 'cs-java']
```

### <a name="sapinstdata"></a>`Sap::InstData`

Simple structure containing the type of an instance along with it's instance
number and the host name of the server on which it must run. This may be a
virtual host for servers that can run on multiple nodes.

See sapnote 1751381 for further details on the START profile formats

Alias of

```puppet
Hash[Enum[
    'type',                 # Category of SAP instance
    'number',               # 2 digit SAP instance number
    'host',                 # Hostname or alias for this component
    'legacy_start_profile', # If true assume separate START profiles 
  ], Variant[
    Sap::InstType,
    Sap::InstNumber,
    Stdlib::Host,
    Boolean,
  ]]
```

### <a name="sapinstnumber"></a>`Sap::InstNumber`

The Sap::InstNumber data type.

Alias of

```puppet
Pattern[/[0-9]{2}/]
```

### <a name="sapinsttype"></a>`Sap::InstType`

The Sap::InstType data type.

Alias of

```puppet
Enum['ASCS', 'SCS', 'ERS', 'D', 'J', 'DVEBMGS', 'G', 'W']
```

### <a name="sapmessageserver"></a>`Sap::MessageServer`

Describes the message server information needed for an external connection to
a given SID.

Alias of

```puppet
Hash[Enum[
    'class',         # e.g. cs-abap or cs-java
    'number',        # e.g. '00'
    'port_override', # e.g. '3915', optional and should be used sparingly
  ], Variant[
    Sap::InstClassSapCs,
    Sap::InstNumber,
    Stdlib::Port,
  ]]
```

### <a name="sapmountpointsidmode"></a>`Sap::MountPointSIDMode`

The Sap::MountPointSIDMode data type.

Alias of

```puppet
Variant[Boolean, Enum[
    'user',
  ]]
```

### <a name="sapmountpoints"></a>`Sap::MountPoints`

The Sap::MountPoints data type.

Alias of

```puppet
Hash[Stdlib::AbsolutePath, Hash[
    Enum[
      'file_params',
      'per_sid',
      'pattern_map',
      'pattern_values',
      'count',
      'mount_params',
      'required_files',
    ],
    Variant[
      Hash,
      Sap::MountPointSIDMode,
      Integer,
      Array[Stdlib::AbsolutePath],
    ],
  ]]
```

### <a name="sapnoderole"></a>`Sap::NodeRole`

The Sap::NodeRole data type.

Alias of

```puppet
Enum['app-standard', 'app-scs', 'app-apo-opt', 'db-standard', 'db-livecache']
```

### <a name="sappuppeturl"></a>`Sap::PuppetUrl`

The Sap::PuppetUrl data type.

Alias of

```puppet
Pattern[/(?i:^puppet?:\/\/)/]
```

### <a name="sapreginfo"></a>`Sap::Reginfo`

The Sap::Reginfo data type.

Alias of

```puppet
Hash[Enum[
    'MODE', # mandatory
    'TP',   # mandatory
    'NO',
    'HOST',
    'ACCESS',
    'CANCEL',
  ], Variant[
    Sap::SecRuleMode,
    Sap::TargetProgram,
    Integer,
    Array[Sap::HostRuleEntry],
  ]]
```

### <a name="sapsid"></a>`Sap::SID`

Format of an SAP system ID

#### Examples

##### Valid data

```puppet
ECP
PRD
QAS
SCP
EQ0
```

Alias of

```puppet
Pattern[/\A[A-Z0-9]{3}\z/]
```

### <a name="sapsidclusterdata"></a>`Sap::SIDClusterData`

The Sap::SIDClusterData data type.

Alias of

```puppet
Hash[Sap::SID, Hash[
    Enum['node_roles', 'components'],
    Variant[
      Sap::ClusterNodeRoles,
      Array[Sap::ClusterComponent],
    ],
  ]]
```

### <a name="sapsidconfigentry"></a>`Sap::SIDConfigEntry`

Configuration entry for a SID

Alias of

```puppet
Struct[=>]
```

#### Parameters

The following parameters are available in the `Sap::SIDConfigEntry` data type:

* [`components`](#components)
* [`backend_databases`](#backend_databases)
* [`manage_profiles`](#manage_profiles)
* [`manage_default_profiles`](#manage_default_profiles)
* [`exclude_common_mount_points`](#exclude_common_mount_points)
* [`pattern_map`](#pattern_map)
* [`pattern_values`](#pattern_values)
* [`instances`](#instances)
* [`databases`](#databases)
* [`primary_backend_db`](#primary_backend_db)
* [`default_pfl_entries`](#default_pfl_entries)
* [`sec_info_rules`](#sec_info_rules)
* [`reg_info_rules`](#reg_info_rules)
* [`db2_query_schedules`](#db2_query_schedules)

##### <a name="components"></a>`components`

A list of associated sap applications to enable for this SID (mandatory).

##### <a name="backend_databases"></a>`backend_databases`

Types of databases to which this system connects (mandatory for SIDs with 'base' enabled)

##### <a name="manage_profiles"></a>`manage_profiles`

Flag indicating whether the instance profiles for this SID should be
managed. When true, `instances`, `databases`, and `primary_backend_db`
become mandatory parameters!

##### <a name="manage_default_profiles"></a>`manage_default_profiles`

Flag indicating whether DEFAULT.PFL for this SID should be managed. When
true, it is created, otherwise it is left absent.

##### <a name="exclude_common_mount_points"></a>`exclude_common_mount_points`

Flag indicating if this SID should skip SID specific entries for the
'common' mountpoint. Some system types do not have /sapmnt directories
(e.g. a DAA instance or liveCache).

##### <a name="pattern_map"></a>`pattern_map`

A hash mapping pattern lookup keys to regular expressions which are
unique to this SID. Note that if one is added it must have a
corresponding value added in `pattern_values`.

##### <a name="pattern_values"></a>`pattern_values`

Mapping from a pattern lookup key to the value which should be
provided wherever the corresponding pattern is found.

##### <a name="instances"></a>`instances`

Hash detailing the instances which are part of this SID. Entries
are keyed by `instance class` and contain the corresponding
[Sap::SidInstance](#sapsidinstance) detail.

##### <a name="databases"></a>`databases`

Hash detailing the database parameters for this SID. Entries in
this hash are keyed by `instance class` and contain the parameters
described by [Sap::SidDatabase](#sapsiddatabase).

##### <a name="primary_backend_db"></a>`primary_backend_db`

Specifies the primary database backend used by the application
servers in this instance. This should map to an entry in `databases'

##### <a name="default_pfl_entries"></a>`default_pfl_entries`

Hash of profile keys and the desired value which will be appended
to the end of the DEFAULT.PFL file for this instance. Currently not validated.

##### <a name="sec_info_rules"></a>`sec_info_rules`

Array of Sap::SecInfo entries to be included in the secinfo rule
file for this SID

##### <a name="reg_info_rules"></a>`reg_info_rules`

Array of Sap::RegInfo entries to be included in the reginfo rule
file for this SID.

##### <a name="db2_query_schedules"></a>`db2_query_schedules`

Array of [Sap::Db2QuerySchedule](#sapdb2queryschedule) entries to be deployed
on each database node of this SID.

### <a name="sapsiddatabase"></a>`Sap::SIDDatabase`

The Sap::SIDDatabase data type.

Alias of

```puppet
Variant[Sap::SIDDatabaseDb2]
```

### <a name="sapsiddatabasedb2"></a>`Sap::SIDDatabaseDb2`

Hash describing an instance which is part of a given SAP SID.

Alias of

```puppet
Struct[=>]
```

#### Parameters

The following parameters are available in the `Sap::SIDDatabaseDb2` data type:

* [`ha_mode`](#ha_mode)
* [`nodes`](#nodes)
* [`fcm_base_port`](#fcm_base_port)
* [`com_port`](#com_port)
* [`host`](#host)
* [`hadr_base_port`](#hadr_base_port)

##### <a name="ha_mode"></a>`ha_mode`

Indicates the HA mode used by this database. Valid values are described
by [Sap::DbHaMode](#sapdbhamode)

##### <a name="nodes"></a>`nodes`

An array containing the trusted cert names of each node that is part of
this database cluster.

##### <a name="fcm_base_port"></a>`fcm_base_port`

The TCP/IP port used as a starting point for configuring this instance.
It will be added to the `com_port` to determine the first communication
port for this database.

##### <a name="com_port"></a>`com_port`

The TCP/IP port used to by clients to connect to this database instance.

##### <a name="host"></a>`host`

Virtual name used by clients to connect to the HA/DR DB2 cluster. Mandatory
when ha_mode is set to 'hadr'.

##### <a name="hadr_base_port"></a>`hadr_base_port`

Used in the configuration of DB2 HA settings.

### <a name="sapsidinstance"></a>`Sap::SIDInstance`

Structure detailing the attributes of a given SAP instance class. This is
assigned as a component of a specific SID's sidconfigentry.

Note that the 'host' parameter is optional and will be present only in the
case that this is a service which can move between nodes.

Alias of

```puppet
Hash[Enum[
    'number',               # e.g. '02'
    'types',                # e.g { 'ASCS' => [ node1.example.com, node2.example.com ] }
    'host',                 # e.g. 'ecd-cs'
    'legacy_start_profile', # Boolean
  ], Variant[
    Sap::InstNumber,
    Hash[Sap::InstType, Array[Stdlib::Fqdn]],
    Stdlib::Host,
    Boolean,
  ]]
```

### <a name="sapsapcomponent"></a>`Sap::SapComponent`

Structure representing each valid component which could be installed on an SAP
server. The
* `ads`            - Adobe Document Services
* `base`           - Catch-all for Standard SAP instance functionality
* `base_extended`  - Extra requirements which are shared by several advanced
                    types
* `bo`             - Business Objects
* `cloudconnector` - Cloud Connector relating to HANA somehow
* `common`         - Underlying requirements shared by all instances
* `db2`            - Database server node hosting an IBM DB2 instance
* `experimental`   - special features which are in development
* 'hana'           - Database node hosting a SAP in-memory HANA instance
* 'maxdb'          - Node hosting a maxdb database instance
* 'livecache'      - Database node hosting a liveCache instance
* 'router'         - Node hosting a SAP-router process

Alias of

```puppet
Enum['ads', 'base', 'base_extended', 'bo', 'cloudconnector', 'common', 'db2', 'experimental', 'hana', 'maxdb', 'liveCache', 'liveCache_client', 'router', 'edi']
```

### <a name="sapscheduleentry"></a>`Sap::ScheduleEntry`

The Sap::ScheduleEntry data type.

Alias of

```puppet
Hash[String, Variant[
    String,
    Integer,
    Array,
  ]]
```

### <a name="sapsecrulemode"></a>`Sap::SecRuleMode`

The Sap::SecRuleMode data type.

Alias of

```puppet
Enum['P', 'D']
```

### <a name="sapsecuser"></a>`Sap::SecUser`

The Sap::SecUser data type.

Alias of

```puppet
Pattern[/\A[*]\z/, /\A[A-Za-z0-9]{,11}\z/]
```

### <a name="sapsecinfo"></a>`Sap::Secinfo`

The Sap::Secinfo data type.

Alias of

```puppet
Hash[Enum[
    'MODE', # mandatory
    'TP',   # mandatory
    'USER', # mandatory
    'HOST', # mandatory
    'USER-HOST',
  ], Variant[
    Sap::SecRuleMode,
    Sap::TargetProgram,
    Sap::SecUser,
    Array[Sap::HostRuleEntry],
  ]]
```

### <a name="sapsourceurl"></a>`Sap::SourceURL`

The Sap::SourceURL data type.

Alias of

```puppet
Variant[Sap::PuppetUrl, Stdlib::Httpurl, Stdlib::Httpsurl]
```

### <a name="sapsourcedfile"></a>`Sap::SourcedFile`

The Sap::SourcedFile data type.

Alias of

```puppet
Hash[String, Sap::SourceURL]
```

### <a name="saptargetprogram"></a>`Sap::TargetProgram`

The Sap::TargetProgram data type.

Alias of

```puppet
Pattern[/\A[A-Za-z0-9_\-\\\/]{,63}([A-Za-z0-9_\-\\\/]{1}|[*])?\Z/]
```

### <a name="sapvglvname"></a>`Sap::VgLvName`

This artificially restricts vg names to [a-zA-Z0-9_]{2,} for
simplification purposes.

Alias of

```puppet
Pattern[/[a-zA-Z0-9_]{2,}/]
```

